'''
Linear limb darkening coefficients in various bands
Coefficients for moa2r, moa2V, ogleI, ogleV, and cfht_I are the same values used by the Goddard Microlensing group
Coefficients for W149, W146, Z087, R062, F184 and K213 were derived from Claret & Bloemen 2011
https://www.aanda.org/articles/aa/full_html/2011/05/aa16451-11/aa16451-11.html
'''

moa2r = 0.7259
moa2V = 0.6730 
ogleI = 0.6155
ogleV = 0.6730
cfht_I = 0.5090
W149 = 0.3300
W146 = 0.3300 # just a median over K and H bands.
Z087 = 0.4534
R062 = 0.5987
F184 = 0.2914 # JB: I used H-Band. Not quite right, but closest match
K213 = 0.2517

