kappa_mas_over_solar_mass = 8.144
solar_mass__kg = 1.988416e30
earth_mass__kg = 5.9722e24
jupiter_mass_kg = 1.898e27  # upate later
solar_mass__earth_mass = solar_mass__kg/earth_mass__kg
jupiter_mass__earth_mass = jupiter_mass_kg/earth_mass__kg
solar_mass__jupiter_mass = solar_mass__kg/jupiter_mass_kg
